/*
    Copyright (C) 2005-2007  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <string.h>
#include "gui.h"

static char *get_selected_servername(void);
static void build_config_window(ConfigServer config);
static void update_avail_servers(void);

// If strlen(original_servername) > 0 then we are editing a already existing server
static char original_servername[128];

void
on_checkShowNotificationOnDownloadDone_toggled(GtkWidget *caller, gpointer data)
{
    ConfigOpts opts;

    opts = nntpgrab_config_get_opts(core);
    opts.show_notification_on_file_downloaded = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(caller));
    nntpgrab_config_set_opts(core, opts);
}

void
on_checkShowNotificationOnFileDecoded_toggled(GtkWidget *caller, gpointer data)
{
    ConfigOpts opts;

    opts = nntpgrab_config_get_opts(core);
    opts.show_notification_on_file_decoded = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(caller));
    nntpgrab_config_set_opts(core, opts);
}

void
on_btnAddUsenetServer_clicked(GtkWidget *caller, gpointer data)
{
    ConfigServer config;

#if 1   // Version 0.1 doesn't support multiple servers yet
    GtkWidget *windowConfig = get_widget("windowConfig");
    GtkWidget *treeConfigServers = get_widget("treeConfigServers");
    GtkTreeModel *model;
    GtkTreeSelection *selection;
    GtkTreeIter iter;

    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeConfigServers));
    if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
        dialog_show(windowConfig, "This version of NNTPGrab doesn't support multiple servers yet", GTK_MESSAGE_INFO, GTK_BUTTONS_OK);
        return;
    }
#endif

    memset(&config, 0, sizeof(config));

    config.port = 119;
    config.max_threads = 3;
    config.priority = 1;

    build_config_window(config);

    memset(&original_servername, 0, sizeof(original_servername));
}

void
on_btnRemoveUsenetServer_clicked(GtkWidget *caller, gpointer data)
{
    char *servername;

    servername = get_selected_servername();
    g_assert(nntpgrab_config_del_server(core, servername));
    g_free(servername);

    update_avail_servers();
}

void
on_btnEditUsenetServer_clicked(GtkWidget *caller, gpointer data)
{
    ConfigServer config;
    char *servername;

    servername = get_selected_servername();

    g_return_if_fail(nntpgrab_config_get_server_info(core, servername, &config));

    g_free(servername);

    build_config_window(config);

    strncpy(original_servername, config.servername, sizeof(original_servername));
}

static char *
get_selected_servername(void)
{
    GtkWidget *treeConfigServers = get_widget("treeConfigServers");
    GtkTreeModel *model;
    GtkTreeSelection *selection;
    GtkTreeIter iter;
    char *servername;

    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeConfigServers));
    g_assert(gtk_tree_selection_get_selected(selection, &model, &iter));

    gtk_tree_model_get(model, &iter, 0, &servername, -1);

    return servername;
}

void
on_btnConfigSave_clicked(GtkWidget *caller, gpointer data)
{
    GtkWidget *windowConfig = get_widget("windowConfig");
    ConfigServer config;
    GtkWidget *txtConfigName;
    GtkWidget *txtConfigHostname;
    GtkWidget *chkConfigServerRequiresAuthentication;
    GtkWidget *txtConfigUsername;
    GtkWidget *txtConfigPassword;
    GtkWidget *spinConfigPort;
    GtkWidget *spinConfigMaxThreads;
    GtkWidget *chkUseSSL;

    windowConfig = get_widget("windowConfig");
    txtConfigName = get_widget("txtConfigName");
    txtConfigHostname = get_widget("txtConfigHostname");
    chkConfigServerRequiresAuthentication = get_widget("chkConfigServerRequiresAuthentication");
    txtConfigUsername = get_widget("txtConfigUsername");
    txtConfigPassword = get_widget("txtConfigPassword");
    spinConfigPort = get_widget("spinConfigPort");
    spinConfigMaxThreads = get_widget("spinConfigMaxThreads");
    chkUseSSL = get_widget("chkUseSSL");

    memset(&config, 0, sizeof(config));

    strncpy(config.servername, gtk_entry_get_text(GTK_ENTRY(txtConfigName)), sizeof(config.servername));
    strncpy(config.hostname, gtk_entry_get_text(GTK_ENTRY(txtConfigHostname)), sizeof(config.hostname));
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(chkConfigServerRequiresAuthentication))) {
        strncpy(config.username, gtk_entry_get_text(GTK_ENTRY(txtConfigUsername)), sizeof(config.username));
        strncpy(config.password, gtk_entry_get_text(GTK_ENTRY(txtConfigPassword)), sizeof(config.password));
    }
    config.port = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinConfigPort));
    config.max_threads = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinConfigMaxThreads));
    config.use_ssl = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(chkUseSSL));

    if (strlen(original_servername) > 0) {
        g_assert(nntpgrab_config_edit_server(core, original_servername, config));
    } else {
        nntpgrab_config_add_server(core, config);
    }

    gtk_widget_hide(windowConfig);

    update_avail_servers();
}

void
on_btnConfigCancel_clicked(GtkWidget *caller, gpointer data)
{
    GtkWidget *windowConfig = get_widget("windowConfig");

    gtk_widget_hide(windowConfig);
}

gboolean
on_windowConfig_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
    int retval = dialog_show(widget, _("Do you want to save changes?"), GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO | GTK_BUTTONS_CANCEL);

    switch (retval) {
        case GTK_RESPONSE_CANCEL:
            break;

        case GTK_RESPONSE_YES:
            on_btnConfigSave_clicked(widget, user_data);
            break;

        case GTK_RESPONSE_NO:
            on_btnConfigCancel_clicked(widget, user_data);
            break;

        default:
            g_error(_("Invalid return value: %i\n"), retval);
    }

    return TRUE;
}

void
on_chkConfigServerRequiresAuthentication_toggled(GtkToggleButton *caller, gpointer data)
{
    GtkWidget *frameConfigAuthentication = get_widget("frameConfigAuthentication");

    if (gtk_toggle_button_get_active(caller)) {
        gtk_widget_set_sensitive(frameConfigAuthentication, TRUE);
    } else {
        gtk_widget_set_sensitive(frameConfigAuthentication, FALSE);
    }
}

void
on_filechooserConfigDownloadDirectory_current_folder_changed(GtkWidget *caller, gpointer data)
{
    char *folder;
    char *tmp;
    ConfigOpts opts;

    folder = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(caller));

    opts = nntpgrab_config_get_opts(core);

    // Strip the 'file://' part of the URI
    if (strstr(folder, "file://")) {
        tmp = folder + 7;
    } else {
        tmp = folder;
    }

    if (!strcmp(tmp, opts.download_directory)) {
        // Nothing changed, ignore
        return;
    }

    snprintf((char *) opts.download_directory, sizeof(opts.download_directory), "%s", tmp);

    g_free(folder);

    nntpgrab_config_set_opts(core, opts);
}

void
on_filechooserConfigTempDirectory_current_folder_changed(GtkWidget *caller, gpointer data)
{
    char *folder;
    char *tmp;
    ConfigOpts opts;

    folder = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(caller));

    opts = nntpgrab_config_get_opts(core);

    // Strip the 'file://' part of the URI
    if (strstr(folder, "file://")) {
        tmp = folder + 7;
    } else {
        tmp = folder;
    }

    if (!strcmp(tmp, opts.temp_directory)) {
        // Nothing changed, ignore
        return;
    }

    snprintf((char *) opts.temp_directory, sizeof(opts.temp_directory), "%s", tmp);

    g_free(folder);

    nntpgrab_config_set_opts(core, opts);
}

static void
on_selection_changed(GtkTreeSelection *selection, gpointer data)
{
    GtkWidget *btnRemoveUsenetServer;
    GtkWidget *btnEditUsenetServer;

    btnRemoveUsenetServer = get_widget("btnRemoveUsenetServer");
    btnEditUsenetServer = get_widget("btnEditUsenetServer");

    if (gtk_tree_selection_count_selected_rows(selection) == 0) {
        gtk_widget_set_sensitive(btnRemoveUsenetServer, FALSE);
        gtk_widget_set_sensitive(btnEditUsenetServer, FALSE);
    } else {
        gtk_widget_set_sensitive(btnRemoveUsenetServer, TRUE);
        gtk_widget_set_sensitive(btnEditUsenetServer, TRUE);
    }
}

static void
build_config_window(ConfigServer config)
{
    GtkWidget *windowConfig;
    GtkWidget *txtConfigName;
    GtkWidget *txtConfigHostname;
    GtkWidget *chkConfigServerRequiresAuthentication;
    GtkWidget *txtConfigUsername;
    GtkWidget *txtConfigPassword;
    GtkWidget *spinConfigPort;
    GtkWidget *spinConfigMaxThreads;
    GtkWidget *chkUseSSL;

    windowConfig = get_widget("windowConfig");
    txtConfigName = get_widget("txtConfigName");
    txtConfigHostname = get_widget("txtConfigHostname");
    chkConfigServerRequiresAuthentication = get_widget("chkConfigServerRequiresAuthentication");
    txtConfigUsername = get_widget("txtConfigUsername");
    txtConfigPassword = get_widget("txtConfigPassword");
    spinConfigPort = get_widget("spinConfigPort");
    spinConfigMaxThreads = get_widget("spinConfigMaxThreads");
    chkUseSSL = get_widget("chkUseSSL");

    gtk_entry_set_text(GTK_ENTRY(txtConfigName), config.servername);
    gtk_entry_set_text(GTK_ENTRY(txtConfigHostname), config.hostname);
    if (strlen(config.username) > 0) {
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkConfigServerRequiresAuthentication), TRUE);
        gtk_entry_set_text(GTK_ENTRY(txtConfigUsername), config.username);
        gtk_entry_set_text(GTK_ENTRY(txtConfigPassword), config.password);
    } else {
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkConfigServerRequiresAuthentication), FALSE);
        gtk_entry_set_text(GTK_ENTRY(txtConfigUsername), "");
        gtk_entry_set_text(GTK_ENTRY(txtConfigPassword), "");
    }

    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinConfigPort), config.port);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinConfigMaxThreads), config.max_threads);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkUseSSL), config.use_ssl);

    gtk_widget_show(windowConfig);
}

static void
update_avail_servers(void)
{
    GtkWidget *treeConfigServers;
    GtkTreeModel *model;
    GtkTreeSelection *selection;
    GList *list, *servers;
    GtkTreeIter iter;

    treeConfigServers = get_widget("treeConfigServers");
    model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeConfigServers));
    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeConfigServers));

    g_signal_connect(selection, "changed", G_CALLBACK(on_selection_changed), NULL);

    gtk_list_store_clear(GTK_LIST_STORE(model));

    servers = nntpgrab_config_get_avail_servers(core);

    list = servers;
    while (list) {
        const char *servername = (const char *) list->data;
        ConfigServer config;
	
	g_return_if_fail(nntpgrab_config_get_server_info(core, servername, &config));

        gtk_list_store_append(GTK_LIST_STORE(model), &iter);
        gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, config.servername, 1, config.priority, 2, config.max_threads, -1);

        list = g_list_next(list);
    }

    if (gtk_tree_model_get_iter_first(model, &iter)) {
        gtk_tree_selection_select_iter(selection, &iter);
    }

    nntpgrab_config_free_avail_servers(core, servers);
}

void
config_initialize()
{
    GtkWidget *treeConfigServers = get_widget("treeConfigServers");
    GtkTreeSelection *selection;
    GtkListStore *store;
    GtkCellRenderer *cellRenderer;
    GtkWidget *filechooserConfigDownloadDirectory;
    GtkWidget *filechooserConfigTempDirectory;
    GtkWidget *checkShowNotificationOnDownloadDone;
    GtkWidget *checkShowNotificationOnFileDecoded;
    ConfigOpts opts;

    opts = nntpgrab_config_get_opts(core);

    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeConfigServers));
    gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);

    cellRenderer = gtk_cell_renderer_text_new();
    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(treeConfigServers), -1, _("Servername"), cellRenderer, "text", 0, NULL);

#if 0       // Future work
    cellRenderer = gtk_cell_renderer_text_new();
    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(treeConfigServers), -1, _("Priority"), cellRenderer, "text", 1, NULL);
#endif

    cellRenderer = gtk_cell_renderer_text_new();
    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(treeConfigServers), -1, _("Max threads"), cellRenderer, "text", 2, NULL);

    store = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT);

    gtk_tree_view_set_model(GTK_TREE_VIEW(treeConfigServers), GTK_TREE_MODEL(store));

    filechooserConfigDownloadDirectory = get_widget("filechooserConfigDownloadDirectory");
    filechooserConfigTempDirectory = get_widget("filechooserConfigTempDirectory");
    checkShowNotificationOnDownloadDone = get_widget("checkShowNotificationOnDownloadDone");
    checkShowNotificationOnFileDecoded = get_widget("checkShowNotificationOnFileDecoded");

    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(filechooserConfigDownloadDirectory), opts.download_directory);
    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(filechooserConfigTempDirectory), opts.temp_directory);

    g_signal_handlers_block_by_func (G_OBJECT (checkShowNotificationOnDownloadDone), G_CALLBACK (on_checkShowNotificationOnDownloadDone_toggled), NULL);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkShowNotificationOnDownloadDone), opts.show_notification_on_file_downloaded);
    g_signal_handlers_unblock_by_func (G_OBJECT (checkShowNotificationOnDownloadDone), G_CALLBACK (on_checkShowNotificationOnDownloadDone_toggled), NULL);

    g_signal_handlers_block_by_func (G_OBJECT (checkShowNotificationOnFileDecoded), G_CALLBACK (on_checkShowNotificationOnFileDecoded_toggled), NULL);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkShowNotificationOnFileDecoded), opts.show_notification_on_file_decoded);
    g_signal_handlers_unblock_by_func (G_OBJECT (checkShowNotificationOnFileDecoded), G_CALLBACK (on_checkShowNotificationOnFileDecoded_toggled), NULL);

    update_avail_servers();
}
