/*
    Copyright (C) 2005-2007  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdlib.h>
#include <strings.h>
#include "nntpgrab.h"
#include "nntpgrab_regex.h"

gboolean
nntpgrab_utils_strip_subject(const char *subject, char **suffix, int *file_num, int *total_files, char **filename, char **extension, NNTPFileType *file_type, int *part_num, int *total_parts)
{
    const char **values;
    static REGEX *re_filename_and_extension = NULL;

    if (!re_filename_and_extension) {
        re_filename_and_extension = regex_compile("(?:.*[\"'\\]]|[^ ]+ |^)([^\"'\\]]+\\.([fF][lL][aA][cC]|[pP][aA][rR]2|[a-zA-Z0-9]{3}|[tT][sS]))(?:[\"'\\] ].*)\\(([0-9]+)/([0-9]+)\\)$");
    }

    values = regex_match(re_filename_and_extension, subject);
    if (values == NULL) {
        return FALSE;
    }

    *filename = g_strdup(values[1]);
    if (extension) {
        *extension = g_strdup(values[2]);
    }

    regex_matches_free(values);

    /*
        // Sloop het partnummer (x/y)
        re_header = pcre_prepare("(.*) [\\(\\[](\\d+)\\/(\\d+)[\\)\\]].*");

        // Sloop de extensie (alles met een . begin gevolgd door 'flac', 'par2', 'ts' of alle andere combinaties met 3 tekens en afgesloten door een spatie, een ", ' of een ]
        re_extension1 = pcre_prepare("(.*)\\.([fF][lL][aA][cC]|[pP][aA][rR]2|[a-zA-Z0-9]{3}|[tT][sS])([ \"'\\]].*)");

        // Sloop een .partXXX stukkie
        re_extension2 = pcre_prepare("(.*)((\\.[pP][aA][rR][tT][0-9]{1,3})|\\.)$");

        // Sloop een .volX+Y stukkie
        re_extension3 = pcre_prepare("(.*)\\.[vV][oO][lL]\\d+\\+\\d+$");

        // Sloop een [1/5] of [1 of 5] of (1/5) stukkie of 1 of 5 stukkie
        re_collection1 = pcre_prepare("(.*)\\D\\d+[ ofOF\\|/\\\\]{1,5}\\d+\\D(.*)");
    */

    return TRUE;
}

char *
nntpgrab_utils_calculate_file_size(guint64 file_size)
{
    double d_file_size;

    d_file_size = file_size;
    if (d_file_size < 1024) {
        return g_strdup_printf("%.1f bytes", d_file_size);
    }

    d_file_size = file_size;
    d_file_size /= 1024;
    if (d_file_size < 1024) {
        return g_strdup_printf("%.1f KB", d_file_size);
    }

    d_file_size /= 1024;
    if (d_file_size < 1024) {
        return g_strdup_printf("%.1f MB", d_file_size);
    }

    d_file_size /= 1024;
    return g_strdup_printf("%.1f GB", d_file_size);
}
